/*************************************************************************\
    Copyright (c) 2009 Corporation for National Research Initiatives;
                        All rights reserved.
     The CNRI open source license for this software is available at 
                  http://hdl.handle.net/1895.26/1013
---------------------------------------------------------------------------
Developer: Sean Reilly <sreilly@cnri.reston.va.us>
\*************************************************************************/

package net.cnri.apps.doserver.txnlog;

import java.util.*;

/** Class responsible for keeping track of the transaction queue. */
abstract public class AbstractTransactionQueue {
  private Vector queueListeners;
  
  public AbstractTransactionQueue()
  {
    this.queueListeners = new Vector();
  }

  public void addQueueListener(TransactionQueueListener l) {
    queueListeners.addElement(l);
  }

  public void removeQueueListener(TransactionQueueListener l) {
    queueListeners.removeElement(l);
  }
    
  /******************************************************************
   * Notify any objects that are listening for new transactions
   * that the given transaction has been added to the queue.
   ******************************************************************/
  protected void notifyQueueListeners(Transaction txn) {
    for(int i=0; i<queueListeners.size(); i++) {
      try {
        ((TransactionQueueListener)queueListeners.elementAt(i)).transactionAdded(txn);
      } catch (Throwable e) {
        System.err.println("error notifying queue listeners: "+e);
        e.printStackTrace(System.err);
      }
    }
  }
  
  /*******************************************************************************
   * Log the specified transaction to the current queue (creating a new queue, if 
   * necessary
   *******************************************************************************/
  abstract public void addTransaction(Transaction txn) 
    throws Exception;

  abstract public void shutdown();

  /** Returns an Enumeration of Transaction objects starting after the
    * given timestamp.  */
  abstract public Enumeration getScanner(long lastTimestamp) 
    throws Exception;
}